@@ -157,14 +157,18 @@ end
         @test CodeGenerators.jl_default_value(p.definitions["B"].fields[1], ctx) === nothing
         @test CodeGenerators.jl_init_value(p.definitions["B"].fields[1], ctx) == "Ref{A}()"
 
-        s, p, ctx = translate_simple_proto("message A {} message B { optional A a = 1; }", Options(force_required=Dict("main" => Set(["B.a"]))))
-        @test generate_struct_str(p.definitions["B"], ctx) == """
-        struct B
-            a::A
+        @test contains(s, """
+        function PB.encode(e::PB.AbstractProtoEncoder, x::B)
+            initpos = position(e.io)
+            PB.encode(e, 1, x.a)
+            return position(e.io) - initpos
         end
-        """
-        @test CodeGenerators.jl_default_value(p.definitions["B"].fields[1], ctx) === nothing
-        @test CodeGenerators.jl_init_value(p.definitions["B"].fields[1], ctx) == "Ref{A}()"
+        function PB._encoded_size(x::B)
+            encoded_size = 0
+            encoded_size += PB._encoded_size(x.a, 1)
+            return encoded_size
+        end
+        """)
     end
 
     @testset "Struct fields are optional when not marked required" begin
@@ -419,9 +423,8 @@ end
         @test CodeGenerators.jl_init_value(p.definitions["A"].fields[3], ctx) == "Ref{Union{Nothing,C}}(nothing)"
 
         @testset "generated fieldtypes are concrete" begin
-            let mod = Module()
-                Core.eval(mod, Meta.parse(s))
-                A = mod.main_pb.A
+            let _mod = Core.eval(Module(gensym()), Meta.parse(s)).main_pb
+                A = Base.@invokelatest _mod.A
                 for fieldtype in fieldtypes(A)
                     @assert fieldtype >: Nothing # is a Union{Nothing, ...}
                     non_optional_fieldtype = fieldtype.b === Nothing ? fieldtype.a : fieldtype.b
@@ -460,25 +463,24 @@ end
         @test CodeGenerators.jl_init_value(p.definitions["A"].fields[5], ctx) == "Dict{String,C}()"
 
         @testset "Containers inside oneof-specialized structs are not too narrowly typed" begin
-            let mod2 = Module()
-                Core.eval(mod2, Meta.parse(s))
-                A = mod2.main_pb.A
-                B = mod2.main_pb.B
-                C = mod2.main_pb.C
-                a = A()
+            let _mod = Core.eval(Module(gensym()), Meta.parse(s)).main_pb
+                A = Base.@invokelatest _mod.A
+                B = Base.@invokelatest _mod.B
+                C = Base.@invokelatest _mod.C
+                a = invokelatest(A)
                 try
-                    a.d1["a"] = A()
-                    a.d1["aa"] = A(o = OneOf(:a, A()))
-                    a.d1["ab"] = A(o = OneOf(:b, B()))
-                    a.d1["ac"] = A(o = OneOf(:c, C()))
+                    a.d1["a"] = invokelatest(A)
+                    a.d1["aa"] = invokelatest(A; o = OneOf(:a, invokelatest(A)))
+                    a.d1["ab"] = invokelatest(A; o = OneOf(:b, invokelatest(B)))
+                    a.d1["ac"] = invokelatest(A; o = OneOf(:c, invokelatest(C)))
 
-                    a.d2["baa"] = B(a = A(o = OneOf(:a, A())))
-                    a.d2["bab"] = B(a = A(o = OneOf(:b, B())))
-                    a.d2["bac"] = B(a = A(o = OneOf(:c, C())))
+                    a.d2["baa"] = invokelatest(B; a = invokelatest(A; o = OneOf(:a, invokelatest(A))))
+                    a.d2["bab"] = invokelatest(B; a = invokelatest(A; o = OneOf(:b, invokelatest(B))))
+                    a.d2["bac"] = invokelatest(B; a = invokelatest(A; o = OneOf(:c, invokelatest(C))))
 
-                    a.d3["caa"] = C(a = A(o = OneOf(:a, A())))
-                    a.d3["cab"] = C(a = A(o = OneOf(:b, B())))
-                    a.d3["cac"] = C(a = A(o = OneOf(:c, C())))
+                    a.d3["caa"] = invokelatest(C; a = invokelatest(A; o = OneOf(:a, invokelatest(A))))
+                    a.d3["cab"] = invokelatest(C; a = invokelatest(A; o = OneOf(:b, invokelatest(B))))
+                    a.d3["cac"] = invokelatest(C; a = invokelatest(A; o = OneOf(:c, invokelatest(C))))
                     @test true
                 catch e
                     @test false
@@ -938,7 +940,7 @@ end
 
         s, p, ctx = translate_simple_proto("message A { optional float a = 1; }")
         @test CodeGenerators.jl_default_value(p.definitions["A"].fields[1], ctx) == "zero(Float32)"
-        # Test that we compare floats with tripple equals to always serialize negative zero
+        # Test that we compare floats with triple equals to always serialize negative zero
         @test contains(s, """
         function PB.encode(e::PB.AbstractProtoEncoder, x::A)
             initpos = position(e.io)
@@ -953,7 +955,7 @@ end
         """)
         s, p, ctx = translate_simple_proto("message A { optional double a = 1; }")
         @test CodeGenerators.jl_default_value(p.definitions["A"].fields[1], ctx) == "zero(Float64)"
-        # Test that we compare floats with tripple equals to always serialize negative zero
+        # Test that we compare floats with triple equals to always serialize negative zero
         @test contains(s, """
         function PB.encode(e::PB.AbstractProtoEncoder, x::A)
             initpos = position(e.io)
@@ -967,6 +969,104 @@ end
         end
         """)
 
+            s, p, ctx = translate_simple_proto("""
+            syntax = "proto2";
+            message A { repeated int32 c = 3; repeated B f = 6; }
+            message B {}
+            """)
+
+        @testset "Required fields are sent over the wire uncoditionally" begin
+            s, p, ctx = translate_simple_proto("""
+            syntax = "proto2";
+            message A {
+                required int32 req_int = 1;
+                optional int32 opt_int = 2;
+                repeated int32 rep_int = 3;
+                optional B     opt_msg = 4;
+                required B     req_msg = 5;
+                repeated B     rep_msg = 6;
+            }
+            message B {}
+            """)
+            @test contains(s, """
+            struct A
+                req_int::Int32
+                opt_int::Int32
+                rep_int::Vector{Int32}
+                opt_msg::Union{Nothing,B}
+                req_msg::B
+                rep_msg::Vector{B}
+            end
+            """)
+            @test contains(s, """
+            function PB.encode(e::PB.AbstractProtoEncoder, x::A)
+                initpos = position(e.io)
+                PB.encode(e, 1, x.req_int)
+                x.opt_int != zero(Int32) && PB.encode(e, 2, x.opt_int)
+                !isempty(x.rep_int) && PB.encode(e, 3, x.rep_int)
+                !isnothing(x.opt_msg) && PB.encode(e, 4, x.opt_msg)
+                PB.encode(e, 5, x.req_msg)
+                !isempty(x.rep_msg) && PB.encode(e, 6, x.rep_msg)
+                return position(e.io) - initpos
+            end
+            function PB._encoded_size(x::A)
+                encoded_size = 0
+                encoded_size += PB._encoded_size(x.req_int, 1)
+                x.opt_int != zero(Int32) && (encoded_size += PB._encoded_size(x.opt_int, 2))
+                !isempty(x.rep_int) && (encoded_size += PB._encoded_size(x.rep_int, 3))
+                !isnothing(x.opt_msg) && (encoded_size += PB._encoded_size(x.opt_msg, 4))
+                encoded_size += PB._encoded_size(x.req_msg, 5)
+                !isempty(x.rep_msg) && (encoded_size += PB._encoded_size(x.rep_msg, 6))
+                return encoded_size
+            end
+            """)
+
+            s, p, ctx = translate_simple_proto("""
+            syntax = "proto3";
+            message A {
+                         int32 def_int = 1;
+                optional int32 opt_int = 2;
+                repeated int32 rep_int = 3;
+                         B     def_msg = 4;
+                optional B     opt_msg = 5;
+                repeated B     rep_msg = 6;
+            }
+            message B {}
+            """)
+            @test contains(s, """
+            struct A
+                def_int::Int32
+                opt_int::Int32
+                rep_int::Vector{Int32}
+                def_msg::Union{Nothing,B}
+                opt_msg::Union{Nothing,B}
+                rep_msg::Vector{B}
+            end
+            """)
+            @test contains(s, """
+            function PB.encode(e::PB.AbstractProtoEncoder, x::A)
+                initpos = position(e.io)
+                x.def_int != zero(Int32) && PB.encode(e, 1, x.def_int)
+                x.opt_int != zero(Int32) && PB.encode(e, 2, x.opt_int)
+                !isempty(x.rep_int) && PB.encode(e, 3, x.rep_int)
+                !isnothing(x.def_msg) && PB.encode(e, 4, x.def_msg)
+                !isnothing(x.opt_msg) && PB.encode(e, 5, x.opt_msg)
+                !isempty(x.rep_msg) && PB.encode(e, 6, x.rep_msg)
+                return position(e.io) - initpos
+            end
+            function PB._encoded_size(x::A)
+                encoded_size = 0
+                x.def_int != zero(Int32) && (encoded_size += PB._encoded_size(x.def_int, 1))
+                x.opt_int != zero(Int32) && (encoded_size += PB._encoded_size(x.opt_int, 2))
+                !isempty(x.rep_int) && (encoded_size += PB._encoded_size(x.rep_int, 3))
+                !isnothing(x.def_msg) && (encoded_size += PB._encoded_size(x.def_msg, 4))
+                !isnothing(x.opt_msg) && (encoded_size += PB._encoded_size(x.opt_msg, 5))
+                !isempty(x.rep_msg) && (encoded_size += PB._encoded_size(x.rep_msg, 6))
+                return encoded_size
+            end
+            """)
+        end
+
         s, p, ctx = translate_simple_proto("message A { optional string a = 1; }")
         @test CodeGenerators.jl_default_value(p.definitions["A"].fields[1], ctx) == "\"\""
         s, p, ctx = translate_simple_proto("message A { optional bytes a = 1; }")
@@ -1158,14 +1258,16 @@ end
             @test strify(CodeGenerators.maybe_generate_oneof_field_types_method, p.definitions["A"], ctx) == ""
             @test strify(CodeGenerators.maybe_generate_field_numbers_method, p.definitions["A"]) == ""
 
-            mod = Module()
-            Core.eval(mod, Meta.parse(s))
-            @test reserved_fields(mod.main_pb.A) == (names = String[], numbers = Union{Int,UnitRange{Int}}[])
-            @test reserved_fields(mod.main_pb.Foo.T) == (names = String[], numbers = Union{Int,UnitRange{Int}}[])
-            @test extendable_field_numbers(mod.main_pb.A) == Union{Int,UnitRange{Int}}[]
-            @test default_values(mod.main_pb.A) == (;)
-            @test oneof_field_types(mod.main_pb.A) == (;)
-            @test field_numbers(mod.main_pb.A) == (;)
+            let _mod = Core.eval(Module(gensym()), Meta.parse(s)).main_pb
+                A = Base.@invokelatest(_mod.A)
+                Foo_T = Base.@invokelatest(Base.@invokelatest(_mod.Foo).T)
+                @test reserved_fields(A) == (names = String[], numbers = Union{Int,UnitRange{Int}}[])
+                @test reserved_fields(Foo_T) == (names = String[], numbers = Union{Int,UnitRange{Int}}[])
+                @test extendable_field_numbers(A) == Union{Int,UnitRange{Int}}[]
+                @test default_values(A) == (;)
+                @test oneof_field_types(A) == (;)
+                @test field_numbers(A) == (;)
+            end
         end
 
         @testset "metadata_methods are generated when needed" begin
@@ -1204,4 +1306,12 @@ end
         );
         @test  d["main"].proto_file.definitions["FromA"].fields[1].type.package_namespace == "var\"#A\".B"
     end
+
+    @testset "Services are excluded from exports (as they are not currently supported)" begin
+        s, p, ctx = translate_simple_proto("""
+        message A {}
+        service S { rpc foo(A) returns (A); }
+        """)
+        @test contains(s, "export A\n")
+    end
 end
